package TP2;


import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.postgresql.ds.PGPoolingDataSource;

public class ManageUserServlet extends HttpServlet implements UserDAO {

	
	private static Logger _log = null;
	private PGPoolingDataSource source = null;
	
	String explanation;
	
	
	public void init() throws ServletException {
		//get the path to the loggerPropertie file inside the servlet
	    String loggerProperties = this.getServletContext().getInitParameter("loggerProperties");
		loggerProperties = getServletContext().getRealPath("/") + loggerProperties;
		//load the configuration for this application's loggers using the
		// servletLog.properties file
		PropertyConfigurator.configure(loggerProperties);
		//create the logger for this servlet class
		//it will use the configuration for the logger com.java2s.LoggerServlet
		//or inherit from the logger com.java2s if one exists, and so on
		_log = Logger.getLogger(this.getClass());


		_log.info(this.getClass().getName()+" started.");
		 //display a DEBUG level message
	    _log.debug("Sending a DEBUG message");
	    //display an INFO level message
	    _log.info("Sending an INFO message");
	    //display an ERROR level message
	    _log.error("Sending an ERROR message");
	    
	    String database = this.getInitParameter("database");
		String login = this.getServletContext().getInitParameter("login");
		String password = this.getServletContext().getInitParameter("password");
		String server = this.getInitParameter("server");
		String maxConnections = this.getInitParameter("maxConnections");
		
		
		source = new PGPoolingDataSource();
		source.setDataSourceName("ServletDataSource");
		source.setServerName(server);
		source.setDatabaseName(database);
		source.setUser(login);
		source.setPassword(password);
		source.setMaxConnections(Integer.parseInt(maxConnections));
		_log.info("Opened a connectivity on DB : "+source.getDataSourceName());
		
	}

	public void destroy() {
		
		_log.info("Closing the connection pool : "+source.getDataSourceName());
		source.close();
		_log.info("Closed the connection ");

	}

	public void doGet(HttpServletRequest request, HttpServletResponse response)
			throws IOException {
				
		User currentUser = (User) request.getSession().getAttribute("currentUser");
		_log.info(currentUser +"wants to make this action "+request.getParameter("action"));
		
		if (request.getParameter("action").equals("list")){
			_log.info("listing");

			ArrayList<User> users = allUsers();
			request.setAttribute("userList", users);
			//set an error message if necessary
			if (users.isEmpty()) {
				request.setAttribute("errorMessage", "No user in database "+source.getDatabaseName());
			} 
		}
		if (request.getParameter("action").equals("search")){
			_log.info("searching");
			String userToFind = request.getParameter("login");
			ArrayList<User> users = this.findUser(userToFind);
			request.setAttribute("userList", users);
			//set an error message if necessary
			if (users.isEmpty()) {
				request.setAttribute("errorMessage", "No user with this login in database "+source.getDatabaseName());
			} 
		}
		if (request.getParameter("action").equals("add")){
			_log.info("adding");
			User u = new User();
			u.setLogin(request.getParameter("login"));
			u.setPassword(request.getParameter("password"));
			u.setAdmin(Boolean.parseBoolean(request.getParameter("admin")));
			boolean ok = insertUser(u);
			ArrayList<User> users = allUsers();
			request.setAttribute("userList", users);
			//set an error message if necessary
			if (!ok) {
				request.setAttribute("errorMessage", "Failed to create user "+u+" in database "+source.getDatabaseName());
				request.setAttribute("explanation", explanation);
			} 
		}
		if (request.getParameter("action").equals("delete")){
			_log.info("deleting");
			User u = new User();
			u.setLogin(request.getParameter("login"));
			
			boolean ok = deleteUser(u);
			ArrayList<User> users = allUsers();
			request.setAttribute("userList", users);
			//set an error message if necessary
			if (!ok) {
				request.setAttribute("errorMessage", "Failed to delete user "+u+" in database "+source.getDatabaseName());
				request.setAttribute("explanation", explanation);
			} 
		}
		

		//forward to gestion
		try {
			request.getRequestDispatcher("gestion.jsp").forward(request, response);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		
	}

	public boolean deleteUser(User u) {
		boolean ok = false;
		Connection conn = null;
		try {
			conn = source.getConnection();
			conn.setAutoCommit(false);
			
			String query = "DELETE FROM gens WHERE login='"+u.getLogin()+"'";
			Statement stmt = conn.createStatement();
			int nbUpdates = stmt.executeUpdate(query);
			if (nbUpdates==1) ok = true;
			stmt.close();
			conn.commit();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			_log.error(e.getMessage());
			explanation = e.getMessage();
			try {
				conn.rollback();
			} catch (SQLException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
				_log.error(e1.getMessage());
			}
		} finally {
			if (conn != null) {
				try { 
					conn.close(); 
				} catch (SQLException e) {
					_log.error(e.getMessage());
				}
			}
	    }
		return ok;
	}

	public boolean insertUser(User u) {
		boolean ok = false;
		Connection conn = null;
		try {
			conn = source.getConnection();
			conn.setAutoCommit(false);
			
			String query = "INSERT INTO gens VALUES ('"+u.getLogin()+"','"+u.getPassword()+"',"+u.isAdmin()+")";
			Statement stmt = conn.createStatement();
			int nbUpdates = stmt.executeUpdate(query);
			if (nbUpdates==1) ok = true;
			stmt.close();
			conn.commit();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			_log.error(e.getMessage());
			explanation = e.getMessage();
			try {
				conn.rollback();
			} catch (SQLException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
				_log.error(e1.getMessage());
			}
		} finally {
			if (conn != null) {
				try { 
					conn.close(); 
				} catch (SQLException e) {
					_log.error(e.getMessage());
				}
			}
	    }
		return ok;		
	}

	public ArrayList<User> allUsers() {
		ArrayList<User> allUsers = new ArrayList<User>();
		Connection conn = null;
		try {
			conn = source.getConnection();
			conn.setAutoCommit(false);
			
			String query = "SELECT * from gens";
			Statement stmt = conn.createStatement();
			ResultSet rs =  stmt.executeQuery(query);

			while (rs.next()) {
				String login = rs.getString("login");
	   			String password = rs.getString("password");
	   			boolean isAdmin = rs.getBoolean("admin");
	   			User u = new User();
	   			u.setLogin(login);
	   			u.setPassword(password);
	   			u.setAdmin(isAdmin);
	   			allUsers.add(u);
			}

			rs.close();
			stmt.close();
			conn.commit();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			_log.error(e.getMessage());
			explanation = e.getMessage();

			try {
				conn.rollback();
			} catch (SQLException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
				_log.error(e1.getMessage());
			}
		} finally {
			if (conn != null) {
				try { 
					conn.close(); 
				} catch (SQLException e) {
					_log.error(e.getMessage());
				}
			}
	    }
		return allUsers;		
	}

	/**
	 * Do the user update on the database
	 * @param u : User to update in the database Servlet	
	 * @param password : new password of the user
	 * @return the number of update. 1 is OK, else, something is wrong
	 */
	public boolean updateUser(User u ){
		boolean ok = false;
		Connection conn = null;
		try {
			conn = source.getConnection();
			conn.setAutoCommit(false);
			
			String query = "UPDATE gens SET password='"+u.getPassword()+"' WHERE login='"+u.getLogin()+"'";
			Statement stmt = conn.createStatement();
			int nbUpdates = stmt.executeUpdate(query);
		    if (nbUpdates==1) ok = true;
			stmt.close();
			conn.commit();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			_log.error(e.getMessage());
			explanation = e.getMessage();

			try {
				conn.rollback();
			} catch (SQLException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
				_log.error(e1.getMessage());
			}
		} finally {
			if (conn != null) {
				try { 
					conn.close(); 
				} catch (SQLException e) {
					_log.error(e.getMessage());
				}
			}
	    }
		return ok;

	}
	
	public ArrayList<User> findUser(String loginToFind){
		//	_log.info(loginToFind);

		ArrayList<User> allUsers = new ArrayList<User>();
		Connection conn = null;
		try {
			conn = source.getConnection();
			conn.setAutoCommit(false);
			
			String query = "SELECT * from gens where login='"+loginToFind+"'";
   			//_log.info(query);

			Statement stmt = conn.createStatement();
			ResultSet rs =  stmt.executeQuery(query);

			while (rs.next()) {
				String login = rs.getString("login");
	   			String password = rs.getString("password");
	   			boolean isAdmin = rs.getBoolean("admin");
	   			User u = new User();
	   			u.setLogin(login);
	   			u.setPassword(password);
	   			u.setAdmin(isAdmin);
	   			allUsers.add(u);
	   			_log.info("found one user "+u);
			}

			rs.close();
			stmt.close();
			conn.commit();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			_log.error(e.getMessage());
			explanation = e.getMessage();

			try {
				conn.rollback();
			} catch (SQLException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
				_log.error(e1.getMessage());
			}
		} finally {
			if (conn != null) {
				try { 
					conn.close(); 
				} catch (SQLException e) {
					_log.error(e.getMessage());
				}
			}
	    }
		
		return allUsers;
	}
	
	public void doPost(HttpServletRequest request, HttpServletResponse response)
			throws IOException {
		doGet(request, response);
	}


}
