package TPExamen;


import java.io.IOException;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;
import org.postgresql.ds.PGPoolingDataSource;

public class MonQuartierServlet extends HttpServlet {

	
	private static Logger _log = null;
	private PGPoolingDataSource source = null;
	
	String explanation;
	
	
	public void init() throws ServletException {
		//get the path to the loggerPropertie file inside the servlet
	    String loggerProperties = this.getServletContext().getInitParameter("loggerProperties");
		loggerProperties = getServletContext().getRealPath("/") + loggerProperties;
		//load the configuration for this application's loggers using the
		// servletLog.properties file
		PropertyConfigurator.configure(loggerProperties);
		//create the logger for this servlet class
		//it will use the configuration for the logger com.java2s.LoggerServlet
		//or inherit from the logger com.java2s if one exists, and so on
		_log = Logger.getLogger(this.getClass());


		_log.info(this.getClass().getName()+" started.");
		 //display a DEBUG level message
	    _log.debug("Sending a DEBUG message");
	    //display an INFO level message
	    _log.info("Sending an INFO message");
	    //display an ERROR level message
	    _log.error("Sending an ERROR message");
	    
	    String database = this.getInitParameter("database");
		String login = this.getServletContext().getInitParameter("login");
		String password = this.getServletContext().getInitParameter("password");
		String server = this.getInitParameter("server");
		String maxConnections = this.getInitParameter("maxConnections");
		
		
		source = new PGPoolingDataSource();
		source.setDataSourceName("ServletDataSource_"+this.getServletName());
		source.setServerName(server);
		source.setDatabaseName(database);
		source.setUser(login);
		source.setPassword(password);
		source.setMaxConnections(Integer.parseInt(maxConnections));
		_log.info("Opened a connectivity on DB : "+source.getDataSourceName());
		
	}

	public void destroy() {
		
		_log.info("Closing the connection pool : "+source.getDataSourceName());
		source.close();
		_log.info("Closed the connection ");

	}

	public void doGet(HttpServletRequest request, HttpServletResponse response)
			throws IOException {
				
		
		
		
		//récupérer le nom du quartier choisi
		//TODO
		String nomQuartier = request.getParameter("quartierChoisi");
		_log.info("quartierChoisi : "+nomQuartier);
		
		//Récupérez le bean initialisé à partir de la lecture de la base de données
		Quartier q = getQuartierDescription(nomQuartier);
		_log.info("lecture :"+q);
		
		//Placez ce bean dans le scope request
		//TODO
		request.setAttribute("monQuartier", q);
		
		//forward to MonQuartier.jsp
		//TODO
		try {
			request.getRequestDispatcher("MonQuartier.jsp").forward(request, response);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
		
		
	
		
	}

	

	
	/**
	 * Récupère la description du quartier à partir de requètes sur la base de données
	 * @param name : nom du quartier passé en paramètre de la requête
	 * @return un bean de type Quartier, initialisé avec les valeurs lues dans la base de données
	 */
	public Quartier getQuartierDescription(String name) {
		Quartier q = new Quartier();
		q.setName(name);
		Connection conn = null;
		try {
			//récupérer une connection depuis la source
			conn = source.getConnection();
			//passer en mode transactionnel sur la connection:
			conn.setAutoCommit(false);
			
			/*********************** Première requete *************************/
			//TODO
			
			//écrivez votre requète SQL
			String query = "select  count(*) from sports, quartiers where intersects(sports.the_geom, quartiers.the_geom) and sports.nature='Bassin de natation' and quartiers.quartier='"+name+"'";
			//créer un objet Statement stmt
			
			Statement stmt = conn.createStatement();
			//faites exécuter la requète, et récupérer le résultat dans ResultSet rs
			ResultSet rs =  stmt.executeQuery(query);
			int nbPiscines = 0;
			while (rs.next()) {
				//récupérer le compte des piscines
				nbPiscines = rs.getInt(1);
			}
			//conserver le nombre de piscines dans le quartier
			q.setNbPiscines(nbPiscines);
			_log.info("nbPiscines : " + q.getNbPiscines());
			
			//fermer rs
			rs.close();
			//fermer stmt
			stmt.close();

			/*********************** Seconde requete *************************/
			//TODO
			ArrayList<String> listRues = new ArrayList<String>();
			int nbRues = 0;
			//écrivez votre requète SQL
			String query2 = "select distinct (routes.nom_rue_d) from routes, quartiers where intersects(routes.the_geom, quartiers.the_geom) and quartiers.quartier = '"+name+"'";
			//créer un objet Statement stmt
			Statement stmt2 = conn.createStatement();
			//faites exécuter la requète, et récupérer le résultat dans ResultSet rs
			ResultSet rs2 =  stmt2.executeQuery(query2);

			while (rs2.next()) {
				//récupérer le nom du quartier dans le result set
				String nomRue = rs2.getString("nom_rue_d");
				//ajouter la rue à la liste des rues du quartier
				if(nomRue != null) {
					listRues.add(nomRue);
					nbRues++;
				}
				
			}
			//ajouter la liste des rues au quartier
			q.setRues(listRues);
			q.setNbRues(nbRues);
			
			_log.info("nbRues : " + q.getNbRues());
			//fermer rs
			rs2.close();
			//fermer stmt
			stmt2.close();
			

			//sauver les modifications
			conn.commit();
		} catch (SQLException e) {
			e.printStackTrace();
			//faites un log de niveau erreur avec l'exception
			//TODO
			_log.error(e.getMessage());
			explanation = e.getMessage();

			try {
				//annulez la transaction
				conn.rollback();
			} catch (SQLException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
				_log.error(e1.getMessage());
			}
		} finally {
			if (conn != null) {
				try { 
					//fermer la connection
					conn.close(); 
				} catch (SQLException e) {
					_log.error(e.getMessage());
				}
			}
	    }
		return q;		
	}

	
	
	
	
	public void doPost(HttpServletRequest request, HttpServletResponse response)
			throws IOException {
		doGet(request, response);
	}


}
